#define G_LOG_DOMAIN "boilerplate-activatable"

#include "config.h"

#include "boilerplate-activatable.h"
#include "boilerplate-application.h"

#include <glib.h>

/**
 * SECTION:boilerplate-activatable
 * @short_description: Interface for activatable extensions
 * @include: boilerplate-activatable.h
 * @see_also: #PeasExtensionSet
 *
 * #BoilerplateActivatable is an interface which should be implemented by
 * extensions that should be activated on an application.
 **/

G_DEFINE_INTERFACE (BoilerplateActivatable, boilerplate_activatable, G_TYPE_OBJECT)

/**
 * boilerplate_activatable_activate:
 * @activatable: A #BoilerplateActivatable.
 *
 * Activates the extension on the application.
 */
void
boilerplate_activatable_activate (BoilerplateActivatable *activatable)
{
    BoilerplateActivatableInterface *iface;

    g_return_if_fail (BOILERPLATE_IS_ACTIVATABLE (activatable));

    iface = BOILERPLATE_ACTIVATABLE_GET_IFACE (activatable);

    // TODO: Plugins Debug - Remove when done.
    g_print ("Activatable: Activate\n");

    if (iface->activate != NULL)
    {
        iface->activate (activatable);
    }
}

/**
 * boilerplate_activatable_deactivate:
 * @activatable: A #BoilerplateActivatable.
 *
 * Deactivates the extension from the application.
 */
void
boilerplate_activatable_deactivate (BoilerplateActivatable *activatable)
{
    BoilerplateActivatableInterface *iface;

    g_return_if_fail (BOILERPLATE_IS_ACTIVATABLE (activatable));

    iface = BOILERPLATE_ACTIVATABLE_GET_IFACE (activatable);

    // TODO: Plugins Debug - Remove when done.
    g_print ("Activatable: deactivate\n");

    if (iface->deactivate != NULL)
    {
        iface->deactivate (activatable);
    }
}

/**
 * boilerplate_activatable_deactivate:
 * @activatable: A #BoilerplateActivatable.
 *
 * Deactivates the extension from the application.
 */
void
boilerplate_activatable_update_state (BoilerplateActivatable *activatable)
{
    BoilerplateActivatableInterface *iface;

    g_return_if_fail (BOILERPLATE_IS_ACTIVATABLE (activatable));

    iface = BOILERPLATE_ACTIVATABLE_GET_IFACE (activatable);

    // TODO: Plugins Debug - Remove when done.
    g_print ("Activatable: update_state\n");

    if (iface->update_state != NULL)
    {
        iface->update_state (activatable);
    }
}

static void
boilerplate_activatable_default_init (BoilerplateActivatableInterface *iface)
{
	/**
	 * BoilerplateActivatable:app:
	 *
	 * The application property contains the current application for this
	 * #BoilerplateActivatable instance.
	 */
	g_object_interface_install_property (iface,
	                                     g_param_spec_object ("application",
	                                                          "Application",
	                                                          "The application this instance it attached to",
	                                                          BOILERPLATE_TYPE_APPLICATION,
	                                                          G_PARAM_READWRITE |
	                                                          G_PARAM_CONSTRUCT_ONLY |
	                                                          G_PARAM_STATIC_STRINGS));
}
